#----------------------------------------------------------
# Name:     setupAdderInstructions.py
# Author:   Aaron Koch
# Purpose:  Creates adder system for use in the 3D aTAM IU construction
#			
# Summary:	Contains 3 main components:
#				1. DataPaths(7): one for each direction plus Tau
#				2. Adder instructions(1): binary counter which encodes
#						which data to accept and which data to discard.
#							Example: 1000001 would accept Tau and North
#								     1011111 would accept Tau and all directions except Down
#									 0111111 would accept all directions but reject Tau 
#										(TAU SHOULD NEVER BE REJECTED)
#				3. Adders(63): Each adder sums the accepted data and subtracts Tau from 
#							   the final result.
#
#
# TODO: 	MAJOR (Functionality)
#			2. Send outputSignal to correct location
#
#			MINOR (Quality of Life)
#			1. Clean up code, condense if possible
#			2. Take inputs from command line
#				2.1. TDP/TDS Destinations
#				2.2. DataWidth
#----------------------------------------------------------

from TileType import *
from TDPUtilities import *
from TDSUtilities import *
from LocationUtilities import *
from PeriodicCounter import PeriodicCounter

#########################################
#these variables control construction
#	-change these to change the structure
#	 of the adder
#########################################

ICoffset = 65			#Offset for Instruction Counter, should be 65 to ensure the Tau data is always accepted
numAdders = 62 		#Number of times bottom adder increments. try 1, 2, 10, 62 (Full adder = 62)
numDataPaths = 7 	#Number of times top counter increments. 7 = N,S,E,W,U,D,Tau
#tau = 2				#Tau will be encoded in 2's compliment, this is in base 10
inputs = {"N":2, "E":1, "S":1, "W":0, "U":0, "D":1, "Tau":-2}
					#inputs: defines data to be seeded to data paths
verticalOffset = 5
PCoffset = 4

###########################################
# These are intermediate variables 
# 		-Avoid changing if possible
###########################################

#dataWidth = len(bin(tau).replace("0b","")) + 1
tdpFile = "Adder.tdp"
tdsFile = "Adder.tds"
tdpOut = TDP(tdpFile, tdsFile)
tdsOut = TDS(tdsFile)
tdsSeed = TDS("AdderSeed.tds")
tdpSeed = TDP(tdsSeed.name.replace("tds", "tdp"), tdsSeed.name)
make3D = True
seedSet = []
tileSet = []
bitStates = ["0", "1"]
activeStates =  ["a", "na"]
dataPaths = ["N", "E", "S", "W", "U", "D", "Tau"]
DirAbbrevArrayPlusTau = ()
DirAbbrevArrayPlusTau += DirAbbrevArray
DirAbbrevArrayPlusTau +=("Tau",)

firstDataPathHeight = 5 + 2*PCoffset
dataPathHeightDifference = 2*PCoffset+verticalOffset

###########################################
# This code establishes the directions of motion.
# Translating to Daniel Hader's direction paradigm:
#		primaryDirection 	=	 Forward Direction
#		secondaryDirection 	=	 Right Direction
#		tertiaryDirection 	=	 Up Direction
###########################################

primaryDirection = N
primaryDirectionReverse = S
secondaryDirection = E
secondaryDirectionReverse = W
tertiaryDirection = U
tertiaryDirectionReverse = D

#######################################
# The following functions generate the seed for the adder
# For initialization:
#		Not initialized: SeedTau() or seedDataPath()
#		Initialized: generateBinarySeed(), generatePseudoCounterTiles(),
#					 generateDataPathPseudoCounterTiles(), generatePC1Tiles()
#######################################

def condensedSeedTiles(n):
	binary = format(n, "b")[::-1]
	#print binary

	for i, bit in enumerate(binary):
		t = TileType(True)
		t.name = "MainCounterSeed{}".format(i)

		if i == 0:
			t.sides[primaryDirection] = Glue(1, "{}_l_a".format(bit))
			t.sides[secondaryDirectionReverse] = Glue(2, "Left1")
			t.sides[tertiaryDirection] = Glue(2, "L1TurnCommand")
		else:
			t.sides[primaryDirection] = Glue(1, bit)
			if i == len(binary) - 1:
				t.sides[primaryDirection].label += "m"
			if i != len(binary) - 1:
				t.sides[secondaryDirection] = Glue(2, "MainCounterSeed{}".format(i+1))
			if i != 1:
				t.sides[secondaryDirectionReverse] = Glue(2, "MainCounterSeed{}".format(i))

		
		if bit == "1":
			t.color = "Blue"

		if i == 0 or i == 1:
			tdsSeed.AddTile(t)
		tdsOut.AddTile(t)

def AdderConfig(temp):
	config = {}
	config["DATA"] = len(bin(temp).replace("0b","")) + 1
	config["PC2"] = dataWidth + 1
	config["PC1"] = (numDataPaths + 1) * dataPeriod
	config["PC0"] = (numAdders+2) * mainCounterPeriod
	return config

def periodicCounterTiles():
	PC_0 = PeriodicCounter(0, PCoffset-1)
	PC_1 = PeriodicCounter(1, PCoffset-1)
	PC_2 = PeriodicCounter(2, PCoffset-1)

	PC = TDS("PC.tds")
	PC.Merge(PC_0.tds)
	PC.Merge(PC_1.tds)
	PC.Merge(PC_2.tds)
	tdsOut.Merge(PC)
	tdsSeed.AddTiles(PC_0.seedTiles + PC_1.seedTiles + PC_2.seedTiles)

##################################
# The following functions generate worker tiles for the adder
# Modify with extreme caution
##################################

def generateLayer1Tiles():
	generateLayer1LSBTiles()
	generateLayer1MSBTiles()
	generateLayer1IncrementTiles()
	for state in bitStates:
		t = TileType(True)
		t.name = "L1Prop" + state 
		t.sides[primaryDirection] = Glue(1, state) 
		t.sides[primaryDirectionReverse] = Glue(1, state) 
		t.sides[secondaryDirection] = Glue(1, "prop")
		t.sides[secondaryDirectionReverse] = Glue(1, "prop")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1Prop" + state + "na"
		t.sides[primaryDirection] = Glue(1, state + "na") 
		t.sides[primaryDirectionReverse] = Glue(1, state + "na") 
		t.sides[secondaryDirection] = Glue(1, "prop")
		t.sides[secondaryDirectionReverse] = Glue(1, "prop")
		t.sides[tertiaryDirection] = Glue(1, "prop")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1GapRowLSB" + state + "Active"
		t.sides[primaryDirection] = Glue(1, state + "_l")
		t.sides[primaryDirectionReverse] = Glue(1, state + "_l_a")
		t.sides[tertiaryDirectionReverse] = Glue(1, "l2")
		t.sides[tertiaryDirection] = Glue(2, "L2MSBhelper")
		t.sides[secondaryDirection] = Glue(1, "a")
		t.sides[secondaryDirectionReverse] = Glue(1, "LSB")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1GapRowLSB" + state 
		t.sides[primaryDirection] = Glue(1, state + "_l")
		t.sides[primaryDirectionReverse] = Glue(1, state + "_l")
		t.sides[tertiaryDirectionReverse] = Glue(1, "l2")
		t.sides[tertiaryDirection] = Glue(1, "L2_LSB")
		t.sides[secondaryDirection] = Glue(1, "a")
		t.sides[secondaryDirectionReverse] = Glue(1, "LSB")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1GapRowA" + state + "->NA" + state
		t.sides[primaryDirection] = Glue(1, state + "na")
		t.sides[primaryDirectionReverse] = Glue(1, state + "a")
		t.sides[secondaryDirectionReverse] = Glue(1, "a")
		t.sides[secondaryDirection] = Glue(1, "a")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1GapRowNA" + state + "->NA" + state
		t.sides[primaryDirection] = Glue(1, state + "na")
		t.sides[primaryDirectionReverse] = Glue(1, state + "na")
		t.sides[secondaryDirection] = Glue(1, "a")
		t.sides[secondaryDirectionReverse] = Glue(1, "a")
		tdsOut.AddTile(t)


		t = TileType(True)
		t.name = "L1GapRow" + state + "Activate"
		t.sides[primaryDirection] = Glue(1, state + "a")
		t.sides[primaryDirectionReverse] = Glue(1, state) 
		t.sides[secondaryDirectionReverse] = Glue(1, "a")
		t.sides[secondaryDirection] = Glue(1, "prop")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)	

		t = TileType(True)
		t.name = "L1RegActive" + state 
		t.sides[primaryDirection] = Glue(1, state + "a")
		t.sides[primaryDirectionReverse] = Glue(1, state + "a")
		t.sides[secondaryDirectionReverse] = Glue(1, "prop")
		t.sides[secondaryDirection] = Glue(1, "prop")
		t.sides[tertiaryDirection] = Glue(2, state)
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1GapRowMSBActivate" + state
		t.sides[primaryDirection] = Glue(1, state + "m" + "a")
		t.sides[primaryDirectionReverse] = Glue(1, state + "m")
		t.sides[secondaryDirectionReverse] = Glue(1, "a")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1GapRowMSB_A->NA" + state
		t.sides[primaryDirection] = Glue(1, state + "m" + "na")
		t.sides[primaryDirectionReverse] = Glue(1, state + "m" + "a")
		t.sides[secondaryDirectionReverse] = Glue(1, "a")
		t.sides[tertiaryDirection] = Glue(2, "Check0Vertical")
		tdsOut.AddTile(t)

def generateLayer1LSBTiles():
	for state in bitStates:
		t = TileType(True)
		t.name = "L1LSB" + state + "Active"
		t.sides[primaryDirectionReverse] = Glue(1, state + "_l_a")
		t.sides[primaryDirection] = Glue(1, state + "_l_a")
		t.sides[secondaryDirection] = Glue(1, "prop")
		t.sides[secondaryDirectionReverse] = Glue(1, "LSB")
		t.sides[tertiaryDirection] = Glue(1, state + "L2_LSB_Active")
		t.sides[tertiaryDirectionReverse] = Glue(1, "Continue2")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1LSB" + state 
		t.sides[primaryDirection] = Glue(1, state + "_l")
		t.sides[primaryDirectionReverse] = Glue(1, state + "_l")
		t.sides[secondaryDirection] = Glue(1, "prop")
		t.sides[secondaryDirectionReverse] = Glue(1, "LSB")
		t.sides[tertiaryDirection] = Glue(1, "L2_LSB")
		t.sides[tertiaryDirectionReverse] = Glue(1, "Continue2")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

def generateLayer1MSBTiles():
	for state in bitStates:
		t = TileType(True)
		t.name = "L1MSBProp" + state 
		t.sides[primaryDirection] = Glue(1, state + "m")
		t.sides[primaryDirectionReverse] = Glue(1, state + "m")
		t.sides[secondaryDirectionReverse] = Glue(1, "prop")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1MSB" + state + "Active"
		t.sides[primaryDirection] = Glue(1, state + "m" + "a")
		t.sides[primaryDirectionReverse] = Glue(1, state + "m" + "a")
		t.sides[secondaryDirectionReverse] = Glue(1, "prop")
		t.sides[tertiaryDirection] = Glue(2, state)
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L1MSB" + state + "NotActive"
		t.sides[primaryDirection] = Glue(1, state + "m" + "na")
		t.sides[primaryDirectionReverse] = Glue(1, state + "m" + "na")
		t.sides[secondaryDirectionReverse] = Glue(1, "prop")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

def generateLayer1IncrementTiles():
	for state in bitStates:
		t = TileType(make3D)
		t.name = "L1GapRowIncrementLSB" + state
		t.sides[primaryDirectionReverse] = Glue(1, state + "_l")
		if state == "0":
			t.sides[primaryDirection] = Glue(1, "1_l_a")
			t.sides[secondaryDirection] = Glue(1, "nc")
			t.color = "Blue"
		else:
			t.sides[primaryDirection] = Glue(1, "0_l_a")
			t.sides[secondaryDirection] = Glue(1, "c")
		t.sides[tertiaryDirectionReverse] = Glue(1, "l1l2")
		t.sides[tertiaryDirection] = Glue(2, "L1TurnCommand")
		t.sides[secondaryDirectionReverse] = Glue(1, "LSB")
		tdsOut.AddTile(t)

		t = TileType(make3D)
		t.name = "L1GapRowIncrement" + state + "c"
		t.sides[primaryDirectionReverse] = Glue(1, state + "na")
		t.sides[secondaryDirectionReverse] = Glue(1, "c")
		if state == "0":
			t.sides[primaryDirection] = Glue(1, "1")
			t.sides[secondaryDirection] = Glue(1, "nc")
			t.color = "Blue"
		else:
			t.sides[primaryDirection] = Glue(1, "0")
			t.sides[secondaryDirection] = Glue(1, "c")
		tdsOut.AddTile(t)

		t = TileType(make3D)
		t.name = "L1GapRowIncrement" + state + "nc"
		t.sides[primaryDirectionReverse] = Glue(1, state + "na")
		t.sides[secondaryDirectionReverse] = Glue(1, "nc")
		if state == "0":
			t.sides[primaryDirection] = Glue(1, "0")
			t.sides[secondaryDirection] = Glue(1, "nc")
		else:
			t.sides[primaryDirection] = Glue(1, "1")
			t.sides[secondaryDirection] = Glue(1, "nc")
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(make3D)
		t.name = "L1GapRowMSBIncrement" + state + "c"
		t.sides[primaryDirectionReverse] = Glue(1, state + "mna")
		t.sides[secondaryDirectionReverse] = Glue(1, "c")
		if state == "0":
			t.sides[primaryDirection] = Glue(1, "1m")
			t.color = "Blue"
		else:
			t.sides[primaryDirection] = Glue(1, "0m")
		tdsOut.AddTile(t)

		t = TileType(make3D)
		t.name = "L1GapRowMSBIncrement" + state + "nc"
		t.sides[primaryDirectionReverse] = Glue(1, state + "mna")
		t.sides[secondaryDirectionReverse] = Glue(1, "nc")
		if state == "0":
			t.sides[primaryDirection] = Glue(1, "0m")
		else:
			t.sides[primaryDirection] = Glue(1, "1m")
			t.color = "Blue"
		tdsOut.AddTile(t)

def generateLayer2Tiles():
	t = TileType(True)
	t.name = "L2TurnCommand"
	t.sides[primaryDirection] = Glue(1, "L2")
	t.sides[tertiaryDirectionReverse] = Glue(2, "L1TurnCommand")
	t.sides[tertiaryDirection] = Glue(2, "L2TurnCommand")
	seedSet.append((t, [0, 0, 2]))
	tdsOut.AddTile(t)

	t = TileType(True)
	for state in bitStates:
		t = TileType(True)
		t.name = "L2LSB" + state + "l" + "Active"
		t.sides[tertiaryDirectionReverse] = Glue(2, state + "L2_LSB_Active_l")
		t.sides[tertiaryDirection] = Glue(2, "Place" + state + "LSB")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L2LSB" + state + "Active"
		t.sides[tertiaryDirectionReverse] = Glue(1, state + "L2_LSB_Active")
		t.sides[primaryDirectionReverse] = Glue(1, "r")
		t.sides[primaryDirection] = Glue(1, "r")
		t.sides[tertiaryDirection] = Glue(2, "Place" + state + "LSB")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L2LSB" + state + "m" + "Active"
		t.sides[tertiaryDirectionReverse] = Glue(1, state + "L2_LSB_Active")
		t.sides[primaryDirection] = Glue(1, "m")
		t.sides[primaryDirectionReverse] = Glue(1, "r")
		t.sides[tertiaryDirection] = Glue(2, "Place" + state + "mLSB")
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L2" + state + "Active"
		t.sides[tertiaryDirectionReverse] = Glue(2, state) 
		t.sides[secondaryDirectionReverse] = Glue(1, state)
		if(state == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L2" + state + "Placement"
		t.sides[secondaryDirection] = Glue(1, state) 
		t.sides[tertiaryDirectionReverse] = Glue(1, "L2_LSB")
		t.sides[tertiaryDirection] = Glue(2, state + "Place") 
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L2LSB" + state + "Active"
		t.sides[primaryDirection] = Glue(1, "L2")
		t.sides[primaryDirectionReverse] = Glue(1, "L2")
		t.sides[tertiaryDirectionReverse] = Glue(1, state + "L2_LSB_Active")
		t.sides[tertiaryDirection] = Glue(2, state+"L2_LSB" + state + "Place")
		#tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "L2" + state + "prop"
		t.sides[secondaryDirectionReverse] = Glue(1, state)
		t.sides[tertiaryDirectionReverse] = Glue(1, "prop")
		t.sides[secondaryDirection] = Glue(1, state)
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L2Tile0"
	t.sides[primaryDirection] = Glue(1, "L2")
	t.sides[primaryDirectionReverse] = Glue(1, "L2")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L2LSBACtive0mHelper"
	t.sides[primaryDirectionReverse] = Glue(1, "m")
	t.sides[tertiaryDirectionReverse] = Glue(2, "L2MSBhelper")
	t.color = "Cyan"
	tdsOut.AddTile(t)

def generateLayer3Tiles():
	t = TileType(True)
	t.name = "L3TurnCommand"
	t.sides[tertiaryDirectionReverse] = Glue(2, "L2TurnCommand")
	t.sides[tertiaryDirection] = Glue(2, "L3TurnCommand")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L3Placement0"
	t.sides[tertiaryDirectionReverse] = Glue(2, "0Place")
	t.sides[tertiaryDirection] = Glue(2,"L30Place")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L3Placement1"
	t.sides[tertiaryDirectionReverse] = Glue(2, "1Place")
	t.sides[tertiaryDirection] = Glue(1, "acceptData")
	t.color = "Blue"
	tdsOut.AddTile(t)

 	t = TileType(True)
	t.name = "L3Placement0_LSB"
	t.sides[tertiaryDirectionReverse] = Glue(2, "Place0LSB")
	t.sides[tertiaryDirection] = Glue(2, "L30Turn")
 	tdsOut.AddTile(t)

 	t = TileType(True)
	t.name = "L3Placement0m_LSB"
	t.sides[tertiaryDirectionReverse] = Glue(2, "Place0mLSB")
	t.sides[tertiaryDirection] = Glue(2, "L30mTurn")
 	tdsOut.AddTile(t)

 	t = TileType(True)
 	t.name = "L3Placement1_LSB"
 	t.sides[tertiaryDirectionReverse] = Glue(2, "Place1LSB")
 	t.sides[tertiaryDirection] = Glue(1, "acceptData")
 	t.color = "Blue"
 	tdsOut.AddTile(t)

 	t = TileType(True)
 	t.name = "L3Placement1m_LSB"
 	t.sides[tertiaryDirectionReverse] = Glue(2, "Place1mLSB")
 	t.sides[tertiaryDirection] = Glue(1, "acceptData")
 	t.color = "Blue"
 	tdsOut.AddTile(t)

def generateLayer4Tiles():
	generateLayer4TurnTiles()
	t = TileType(True)
	t.name = "L4TurnCommand"
	t.sides[tertiaryDirectionReverse] = Glue(2, "L3TurnCommand")
	t.sides[tertiaryDirection] = Glue(2, "L4TurnCommand")
	t.sides[secondaryDirectionReverse] = Glue(2, "L4TurnStart")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L4TurnStart"
	t.sides[secondaryDirection] = Glue(2, "L4TurnStart")
	t.sides[primaryDirection] = Glue(1, "start")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L4Placement0"
	t.sides[tertiaryDirectionReverse] = Glue(2, "L30Place")
	t.sides[secondaryDirectionReverse] = Glue(1, "0")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L4Turn0"
	t.sides[tertiaryDirectionReverse] = Glue(2, "L30Turn")
	t.sides[secondaryDirectionReverse] = Glue(1, "0")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "L4Turn0m"
	t.sides[tertiaryDirectionReverse] = Glue(2, "L30mTurn")
	t.sides[secondaryDirectionReverse] = Glue(1, "0m")
	tdsOut.AddTile(t)

 	for state in bitStates:
 		t = TileType(True)
 		t.name = "L4Data" + state
 		t.sides[secondaryDirection] = Glue(1, state)
 		t.sides[secondaryDirectionReverse] = Glue(1, state)
 		t.sides[tertiaryDirectionReverse] = Glue(1, "acceptData")
 		if state == "1":
			t.color = "Blue"
 		tdsOut.AddTile(t)

 		t = TileType(True)
 		t.name = "L4Data" + state + "m"
 		t.sides[secondaryDirection] = Glue(1, state + "m")
 		t.sides[secondaryDirectionReverse] = Glue(1, state + "m")
 		t.sides[tertiaryDirectionReverse] = Glue(1, "acceptData")
 		if state == "1":
			t.color = "Blue"
 		tdsOut.AddTile(t)

 		t = TileType(True)
 		t.name = "L4Data" + state + "l"
 		t.sides[secondaryDirection] = Glue(1, state + "l")
 		t.sides[secondaryDirectionReverse] = Glue(1, state + "l")
 		t.sides[tertiaryDirectionReverse] = Glue(1, "acceptData")
 		if state == "1":
			t.color = "Blue"
 		tdsOut.AddTile(t)

 	t = TileType(True)
	t.name = "L4LSBACtive0mHelper"
	t.sides[primaryDirectionReverse] = Glue(1, "msbHelp")
	t.sides[tertiaryDirectionReverse] = Glue(2, "L4MSBhelper")
	t.color = "Cyan"
	tdsOut.AddTile(t)

def generateLayer4TurnTiles():
	for stateIn in bitStates:
		t = TileType(True)
		t.name = "startTurn" + stateIn + "l"
		t.sides[primaryDirectionReverse] = Glue(1, "start")
		t.sides[secondaryDirection] = Glue(1, stateIn + "l")
		t.sides[primaryDirection] = Glue(1, "{}{}Turn".format(stateIn, "l"))
		tdsOut.AddTile(t)

		if stateIn == "0":
			t = TileType(True)
			t.name = "startTurn" + stateIn 
			t.sides[primaryDirectionReverse] = Glue(1, "start")
			t.sides[secondaryDirection] = Glue(1, stateIn)
			t.sides[primaryDirection] = Glue(1, "{}{}Turn".format(stateIn, "l"))
			tdsOut.AddTile(t)

		for stateOut in bitStates:
			for roleIn in ["m", "", "l"]:
				for roleOut in ["m", "", "l"]:
					if roleOut == "l" and roleIn == "":
						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|PassTurn".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirectionReverse] = Glue(1, "{}{}Turn".format(stateOut, roleOut))
						t.sides[primaryDirection] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[secondaryDirectionReverse] = Glue(2, "{}{}Turn".format(stateIn, roleIn))
						t.sides[secondaryDirection] = Glue(1, "{}{}".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|Prop".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirectionReverse] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[primaryDirection] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[secondaryDirection] = Glue(1, "{}{}".format(stateIn, roleIn))
						t.sides[secondaryDirectionReverse] = Glue(1, "{}{}".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

					if roleOut == "l" and roleIn == "m":
						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|Prop".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirectionReverse] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[primaryDirection] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[secondaryDirection] = Glue(1, "{}{}".format(stateIn, roleIn))
						t.sides[secondaryDirectionReverse] = Glue(1, "{}{}".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

					if roleOut == "" and roleIn == "":
						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|PassTurn".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirectionReverse] = Glue(1, "{}{}pTurn".format(stateOut, roleOut))
						t.sides[primaryDirection] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[secondaryDirectionReverse] = Glue(2, "{}{}Turn".format(stateIn, roleOut))
						t.sides[secondaryDirection] = Glue(1, "{}{}".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|Prop".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirectionReverse] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[primaryDirection] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[secondaryDirection] = Glue(1, "{}{}".format(stateIn, roleIn))
						t.sides[secondaryDirectionReverse] = Glue(1, "{}{}".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

						if stateIn == stateOut:
							t = TileType(True)
							t.name = "IN:{}{}|OUT:{}{}|ReceiveTurn".format(stateIn, roleIn, stateOut, roleOut)
							t.sides[primaryDirection] = Glue(1, "{}{}pTurn".format(stateOut, roleOut))
							t.sides[secondaryDirection] = Glue(2, "{}{}Turn".format(stateIn, roleIn))
							if stateOut == "1":
								t.color = "Blue"
							tdsOut.AddTile(t)

					if roleOut == "" and roleIn == "m":
						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|PassTurn".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirectionReverse] = Glue(1, "{}{}pTurn".format(stateOut, roleOut))
						t.sides[primaryDirection] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[secondaryDirectionReverse] = Glue(2, "{}{}Turn".format(stateIn, roleIn))
						t.sides[secondaryDirection] = Glue(1, "{}{}".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|Prop".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirectionReverse] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[primaryDirection] = Glue(1, "{}{}p".format(stateOut, roleOut))
						t.sides[secondaryDirection] = Glue(1, "{}{}".format(stateIn, roleIn))
						t.sides[secondaryDirectionReverse] = Glue(1, "{}{}".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

					if roleOut == "m" and roleIn == "m" and stateIn == stateOut:
						t = TileType(True)
						t.name = "IN:{}{}|OUT:{}{}|ReceiveTurn".format(stateIn, roleIn, stateOut, roleOut)
						t.sides[primaryDirection] = Glue(2, "{}{}Advance".format(stateIn, roleIn))
						t.sides[secondaryDirection] = Glue(2, "{}{}Turn".format(stateIn, roleIn))
						if stateOut == "1":
							t.color = "Blue"
						tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "Advance" + stateIn + "m"
		t.sides[primaryDirectionReverse] = Glue(2, stateIn + "mAdvance")
		t.sides[primaryDirection] = Glue(1, stateIn + "m")
		t.sides[secondaryDirection] = Glue(1, "advance")
		if (stateIn == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)
		
		t = TileType(True)
		t.name = "Advance" + stateIn + "reg"
		t.sides[primaryDirectionReverse] = Glue(1, stateIn + "p")
		t.sides[primaryDirection] = Glue(1, stateIn)
		t.sides[secondaryDirectionReverse] = Glue(1, "advance")
		t.sides[secondaryDirection] = Glue(1, "advance")
		if (stateIn == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)
		
		t = TileType(True)
		t.name = "Advance" + stateIn + "l"
		t.sides[secondaryDirectionReverse] = Glue(1, "advance")
		t.sides[primaryDirectionReverse] = Glue(1, stateIn + "lp")
		t.sides[primaryDirection] = Glue(1, stateIn + "l*")
		if (stateIn == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

def generateAdderTiles():
	for x in bitStates:
		for y in bitStates:
			if(y == "1"):
				yCarry = "c"
			else:
				yCarry = "nc"

			t = TileType(True)
			t.name = "StartAdd" + x + "+" + y
			t.sides[primaryDirectionReverse] = Glue(1, x + "l*")
			t.sides[secondaryDirection] = Glue(1, y)
			if(x == "1" and y == "1"):
				t.sides[primaryDirection] = Glue(1, "0" + "l" + "p")
				t.sides[secondaryDirectionReverse] = Glue(1, "c*")
			elif(x == "1" or y == "1"):
				t.sides[primaryDirection] = Glue(1, "1" + "l" + "p")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc*")
			elif(x == "0" and y == "0"):
				t.sides[primaryDirection] = Glue(1, "0" + "l" + "p")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc*")
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "Add" + x + "+" + y
			t.sides[primaryDirectionReverse] = Glue(1, x + "*")
			t.sides[secondaryDirection] = Glue(1, y)
			if(x == "1" and y == "1"):
				t.sides[primaryDirection] = Glue(1, "0" + "p")
				t.sides[secondaryDirectionReverse] = Glue(1, "c*")
			elif(x == "1" or y == "1"):
				t.sides[primaryDirection] = Glue(1, "1" + "p")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc*")
			elif(x == "0" and y == "0"):
				t.sides[primaryDirection] = Glue(1, "0" + "p")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc*")
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "Add" + x + "m" + "+" + y
			t.sides[primaryDirectionReverse] = Glue(1, x + "m*")
			t.sides[secondaryDirection] = Glue(1, y)
			if(x == "1" and y == "1"):
				t.sides[primaryDirection] = Glue(2, "0mAdvance")
			elif(x == "1" or y == "1"):
				t.sides[primaryDirection] = Glue(2, "1mAdvance")
			elif(x == "0" and y == "0"):
				t.sides[primaryDirection] = Glue(2, "0mAdvance")
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "PassAdd" + x + "+" + yCarry
			t.sides[primaryDirectionReverse] = Glue(1, x)
			t.sides[secondaryDirection] = Glue(1, yCarry + "*")
			if(x == "1" and y == "1"):
				t.sides[primaryDirection] = Glue(1, "0*")
				t.sides[secondaryDirectionReverse] = Glue(1, "c")
			elif(x == "1" or y == "1"):
				t.sides[primaryDirection] = Glue(1, "1*")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			elif(x == "0" and y == "0"):
				t.sides[primaryDirection] = Glue(1, "0*")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "PassAdd" + x + "m" + "+" + yCarry
			t.sides[primaryDirectionReverse] = Glue(1, x + "m")
			t.sides[secondaryDirection] = Glue(1, yCarry + "*")
			if(x == "1" and y == "1"):
				t.sides[primaryDirection] = Glue(1, "0m*")
				t.sides[secondaryDirectionReverse] = Glue(1, "c")
			elif(x == "1" or y == "1"):
				t.sides[primaryDirection] = Glue(1, "1m*")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			elif(x == "0" and y == "0"):
				t.sides[primaryDirection] = Glue(1, "0m*")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "propLSBAnswer:" + x + "_In:" + y
			t.sides[primaryDirectionReverse] = Glue(1, x + "lq")
			t.sides[primaryDirection] = Glue(1, x + "lq")
			t.sides[secondaryDirection] = Glue(1, y)
			t.sides[secondaryDirectionReverse] = Glue(1, y)
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "propAnswer:" + x + "_In:" + y
			t.sides[primaryDirectionReverse] = Glue(1, x + "q")
			t.sides[primaryDirection] = Glue(1, x + "q")
			t.sides[secondaryDirection] = Glue(1, y)
			t.sides[secondaryDirectionReverse] = Glue(1, y)
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "propIn:" + x + "m_" + yCarry
			t.sides[primaryDirectionReverse] = Glue(1, x + "m")
			t.sides[secondaryDirection] = Glue(1, yCarry)
			if(x == "1" and y == "1"):
				t.sides[primaryDirection] = Glue(1, "0m")
				t.sides[secondaryDirectionReverse] = Glue(1, "c")
			elif(x == "1" or y == "1"):
				t.sides[primaryDirection] = Glue(1, "1m")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			elif(x == "0" and y == "0"):
				t.sides[primaryDirection] = Glue(1, "0m")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "propIn:" + x + "_" + yCarry
			t.sides[primaryDirectionReverse] = Glue(1, x)
			t.sides[secondaryDirection] = Glue(1, yCarry)
			if(x == "1" and y == "1"):
				t.sides[primaryDirection] = Glue(1, "0")
				t.sides[secondaryDirectionReverse] = Glue(1, "c")
			elif(x == "1" or y == "1"):
				t.sides[primaryDirection] = Glue(1, "1")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			elif(x == "0" and y == "0"):
				t.sides[primaryDirection] = Glue(1, "0")
				t.sides[secondaryDirectionReverse] = Glue(1, "nc")
			tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "Advance" + x + "reg" + "Adder"
		t.sides[primaryDirectionReverse] = Glue(1, x + "q")
		t.sides[primaryDirection] = Glue(1, x)
		t.sides[secondaryDirectionReverse] = Glue(1, "advance")
		t.sides[secondaryDirection] = Glue(1, "advance")
		if (x == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)
		
		t = TileType(True)
		t.name = "Advance" + x + "l" + "Adder"
		t.sides[secondaryDirectionReverse] = Glue(1, "advance")
		t.sides[primaryDirectionReverse] = Glue(1, x + "lq")
		t.sides[primaryDirection] = Glue(1, x + "l*")
		if (x == "1"):
			t.color = "Blue"
		tdsOut.AddTile(t)

def generateCheckTiles():
	for i in range(3):
		t = TileType(True)
		t.name = "CheckVertical" + str(i)
		t.sides[tertiaryDirectionReverse] = Glue(2, "Check" + str(i) + "Vertical")
		if i != 2:
			t.sides[tertiaryDirection] = Glue(2, "Check" + str(i + 1) + "Vertical")
		else:
			t.sides[primaryDirection] = Glue(2, "CheckExtend")
		tdsOut.AddTile(t)

	for i in range(7):
		t = TileType(True)
		t.name = "CheckHorizontal" + str(i)
		t.sides[secondaryDirectionReverse] = Glue(2, "Check" + str(i + 1) + "Horizontal")
		if i == 0:
			t.sides[primaryDirectionReverse] = Glue(2, "CheckExtend")
		else:
			t.sides[secondaryDirection] = Glue(2, "Check" + str(i) + "Horizontal")
		if i == 6:
			t.sides[secondaryDirectionReverse] = Glue(1, "Check")
		tdsOut.AddTile(t)

	for state in bitStates:
		t = TileType(True)
		t.name = "CheckLSB" + state
		t.sides[primaryDirectionReverse] = Glue(1, state+"l*")
		t.sides[secondaryDirection] = Glue(1, "Check")
		t.sides[secondaryDirectionReverse] = Glue(1, "Check")
		t.sides[primaryDirection] = Glue(1, "CelebrateFirstTurn")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "CheckReg" + state
		t.sides[primaryDirectionReverse] = Glue(1, state)
		t.sides[secondaryDirection] = Glue(1, "Check")
		t.sides[secondaryDirectionReverse] = Glue(1, "Check")
		t.sides[primaryDirection] = Glue(1, "CelebratePass")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "CheckMSB" + state
		t.sides[primaryDirectionReverse] = Glue(1, state + "m")
		t.sides[secondaryDirection] = Glue(1, "Check")
		t.sides[secondaryDirectionReverse] = Glue(1, "Check")
		if state == "0":
			t.sides[primaryDirection] = Glue(2, "CelebrateStart")
		tdsOut.AddTile(t)

	passRole = "Celebrate"
	role = "CelebrateStart"
	t = TileType(True)
	t.name = role
	t.sides[primaryDirectionReverse] = Glue(2, role)
	t.sides[secondaryDirection] = Glue(1, passRole)
	t.color = "Red"
	tdsOut.AddTile(t)

	role = "CelebratePass"
	t = TileType(True)
	t.name = role
	t.sides[primaryDirectionReverse] = Glue(1, role)
	t.sides[secondaryDirection] = Glue(1, passRole)
	t.sides[secondaryDirectionReverse] = Glue(1, passRole)
	t.color = ""
	tdsOut.AddTile(t)

	start = 1
	end = PCoffset
	for n in range(start, end):
		print "{}/{}".format(n, end-1)
		role = "CelebrateFirstFall{}/{}".format(n, end-1)
		t = TileType(True)
		t.name = role
		t.sides[tertiaryDirection] = Glue(2, "CelebrateFirstFall{}/{}".format(n-1, end-1))
		t.sides[tertiaryDirectionReverse] = Glue(2, role)
		if n == start:
			t.sides[secondaryDirectionReverse] = Glue(1, passRole)
			t.sides[primaryDirectionReverse] = Glue(1, "CelebrateFirstTurn")
			t.sides[tertiaryDirection] = Glue()
		elif n == range(start,end)[-2]:
			t.sides[secondaryDirectionReverse] = Glue(2, "CelebrateKeyGen1")
			t.sides[tertiaryDirectionReverse].strength = 1
		elif n == range(start,end)[-1]:
			t.sides[secondaryDirectionReverse] = Glue(1, "CelebrateKey1")
			t.sides[tertiaryDirection].strength = 1
			t.sides[tertiaryDirectionReverse] = Glue(1, "FallConnection")
		t.color = "Red"
		tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "CelebrateKeyGen1"
	t.sides[secondaryDirection] = Glue(2, "CelebrateKeyGen1")
	t.sides[tertiaryDirectionReverse] = Glue(2, "CelebrateKeyGen1")
	t.color = "Green"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "CelebrateKey1"
	t.sides[tertiaryDirection] = Glue(2, "CelebrateKeyGen1")
	t.sides[secondaryDirection] = Glue(1, "CelebrateKey1")
	t.color = "Yellow"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "AdderSuccessProp"
	t.sides[primaryDirection] = Glue(1, "AdderSuccess")
	t.sides[primaryDirectionReverse] = Glue(1, "AdderSuccess")
	t.sides[secondaryDirection] = Glue(1, "LSB")
	t.sides[tertiaryDirection] = Glue(1, "FallConnection")
	t.sides[secondaryDirectionReverse] = Glue(1, "BracketConnectionStart") 
	t.color = "Red"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BracketConnectionStart"
	t.sides[secondaryDirectionReverse] = Glue(1, "Turn_Adder")
	t.sides[primaryDirection] = Glue(1, "Ride_Adder")
	t.sides[secondaryDirection] = Glue(1, "BracketConnectionStart")
	t.sides[primaryDirectionReverse] = Glue(1, "BracketConnectionStart")
	t.sides[tertiaryDirectionReverse] = Glue(2, "BracketConnection")
	t.color = "Red"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BracketConnection"
	t.sides[tertiaryDirectionReverse] = Glue(2, "BracketConnection")
	t.sides[tertiaryDirection] = Glue(2, "BracketConnection")
	t.sides[primaryDirection] = Glue(1, "Ride_Adder")
	t.color = "Red"
	tdsOut.AddTile(t)


#No work done in here (Delete?)
def generateDataPath():
	generateDataPathDataTiles()
	generateDataPathTurnTiles()

def generateDataPathDataTiles():
	for state in bitStates:
		t = TileType(True)
		t.name = "DPDataLSB" + state
		t.sides[primaryDirection] = Glue(1, state)
		t.sides[primaryDirectionReverse] = Glue(1, state)
		t.sides[tertiaryDirectionReverse] = Glue(1, "Continue1")
		t.sides[tertiaryDirection] = Glue(1, "turnHelper" + state)
		t.sides[secondaryDirection] = Glue(1, "propDP")
		t.sides[secondaryDirectionReverse] = Glue(1, "LSB")
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "DPDataLSB" + state + "Turn"
		t.sides[primaryDirection] = Glue(1, state)
		t.sides[primaryDirectionReverse] = Glue(1, state)
		t.sides[tertiaryDirectionReverse] = Glue(1, "l1")
		t.sides[tertiaryDirection] = Glue(2, "startTurn" + state)
		t.sides[secondaryDirection] = Glue(1, "propDP")
		t.sides[secondaryDirectionReverse] = Glue(1, "ZERO")
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "DPDataREG" + state
		t.sides[primaryDirection] = Glue(1, state)
		t.sides[primaryDirectionReverse] = Glue(1, state)
		t.sides[tertiaryDirection] = Glue(1, "turnHelper" + state)
		t.sides[secondaryDirection] = Glue(1, "propDP")
		t.sides[secondaryDirectionReverse] = Glue(1, "propDP")
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "DPDataMSB" + state
		t.sides[primaryDirection] = Glue(1, state + "m")
		t.sides[primaryDirectionReverse] = Glue(1, state + "m")
		t.sides[tertiaryDirection] = Glue(1, "turnHelper" + state + "m")
		t.sides[secondaryDirection] = Glue(1, "propDP")
		t.sides[secondaryDirectionReverse] = Glue(1, "propDP")
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)

def generateDataPathTurnTiles():
	for state in bitStates:
		t = TileType(True)
		t.name = "DPTurnStartPass" + state
		t.sides[tertiaryDirectionReverse] = Glue(2, "startTurn" + state)
		t.sides[secondaryDirectionReverse] = Glue(2, "Left1/1")
		t.sides[primaryDirection] = Glue(1, "startTurn" + state)
		t.color = "Green"
		tdsOut.AddTile(t) 

		t = TileType(True)
		t.name = "DPTurnStartReceive" + state
		t.sides[primaryDirectionReverse] = Glue(1, "startTurn" + state)
		t.sides[tertiaryDirectionReverse] = Glue(1, "turnHelper" + state)
		t.sides[secondaryDirectionReverse] = Glue(1, state + "l")
		t.sides[secondaryDirection] = Glue(1, "passTurn")
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "DPTurnPass" + state
		t.sides[primaryDirection] = Glue(1, "turn")
		t.sides[tertiaryDirectionReverse] = Glue(1, "turnHelper" + state)
		t.sides[secondaryDirectionReverse] = Glue(1, "passTurn")
		t.color = "Cyan"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "DPTurnReceive" + state
		t.sides[primaryDirectionReverse] = Glue(1, "turn")
		t.sides[tertiaryDirectionReverse] = Glue(1, "turnHelper" + state)
		t.sides[secondaryDirectionReverse] = Glue(1, state)
		t.sides[secondaryDirection] = Glue(1, "passTurn")
		if state == "1":
			t.color = "Blue"
		tdsOut.AddTile(t)
		
		t = TileType(True)
		t.name = "DPTurnPass" + state + "m"
		t.sides[primaryDirection] = Glue(1, "turn")
		t.sides[tertiaryDirectionReverse] = Glue(1, "turnHelper" + state + "m")
		t.sides[secondaryDirectionReverse] = Glue(1, "passTurn")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "DPTurnReceive" + state + "m"
		t.sides[primaryDirectionReverse] = Glue(1, "turn")
		t.sides[tertiaryDirectionReverse] = Glue(1, "turnHelper" + state + "m")
		t.sides[secondaryDirectionReverse] = Glue(1, state + "m")
		tdsOut.AddTile(t)

		for state2 in bitStates:
			t = TileType(True)
			t.name = "DPTurnProp" + state + state2
			t.sides[secondaryDirection] = Glue(1, state)
			t.sides[secondaryDirectionReverse] = Glue(1, state)
			t.sides[tertiaryDirectionReverse] = Glue(1, "turnHelper" + state2)
			if state == "1":
				t.color = "Blue"
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "DPTurnProp" + state + "m" + state2
			t.sides[secondaryDirection] = Glue(1, state + "m")
			t.sides[secondaryDirectionReverse] = Glue(1, state + "m")
			t.sides[tertiaryDirectionReverse] = Glue(1, "turnHelper" + state2)
			tdsOut.AddTile(t)

def generateDPGuideFollowTiles():
	for state in bitStates:
		for role in ["m", "", "l"]:
			t = TileType(True)
			t.name = "{}{}HorizontalGuideFollow".format(state, role)
			t.sides[secondaryDirection] = Glue(1, "{}{}".format(state, role))
			t.sides[secondaryDirectionReverse] = Glue(1, "{}{}".format(state, role))
			if role != "m":
				t.sides[primaryDirection] = Glue(1, "GuideHorizontal")
			t.sides[primaryDirectionReverse] = Glue(1, "GuideHorizontal")
			if state == "1":
				t.color = "Blue"
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "{}{}TurnGuideFollow".format(state, role)
			t.sides[tertiaryDirectionReverse] = Glue(1, "{}{}".format(state, role))
			t.sides[secondaryDirection] = Glue(1, "{}{}".format(state, role))
			if role != "m":
				t.sides[primaryDirection] = Glue(1, "GuideTurn")
			t.sides[primaryDirectionReverse] = Glue(1, "GuideTurn")
			if state == "1":
				t.color = "Blue"
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "{}{}GuideFollow".format(state, role)
			t.sides[tertiaryDirectionReverse] = Glue(1, "{}{}".format(state, role))
			t.sides[tertiaryDirection] = Glue(1, "{}{}".format(state, role))
			if role != "m":
				t.sides[primaryDirection] = Glue(1, "Guide")
			t.sides[primaryDirectionReverse] = Glue(1, "Guide")
			if state == "1":
				t.color = "Blue"
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "{}{}Final".format(state, role)
			t.sides[secondaryDirectionReverse] = Glue(1, "{}{}".format(state, ""))
			t.sides[tertiaryDirection] = Glue(1, "{}{}".format(state, role))
			if role != "m":
				t.sides[primaryDirection] = Glue(1, "Final")
			t.sides[primaryDirectionReverse] = Glue(1, "Final")
			if state == "1":
				t.color = "Blue"
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = "{}{}NorthFinal".format(state, role)
			t.sides[secondaryDirectionReverse] = Glue(1, "{}{}".format(state, role))
			t.sides[tertiaryDirection] = Glue(1, "{}{}".format(state, role))
			if role != "m":
				t.sides[primaryDirection] = Glue(1, "NorthFinal")
			t.sides[primaryDirectionReverse] = Glue(1, "NorthFinal")
			if state == "1":
				t.color = "Blue"
			tdsOut.AddTile(t)


def generateDPPlacementTiles():
	vDist = dataPathHeightDifference
	initial = firstDataPathHeight
	totalHeight = initial + numDataPaths * vDist
	print "TotalHeight = {}".format(totalHeight)

	horizontalLeft = 2
	horizontalRight = 2
	totalHorizontal = horizontalLeft + horizontalRight

	for i in range(totalHeight-vDist+1):
		t = TileType(True)
		t.name = "Placement{}/{}".format(i, totalHeight)
		t.sides[tertiaryDirection] = Glue(2, "Placement{}".format(i))
		t.sides[tertiaryDirectionReverse] = Glue(2, "Placement{}".format(i-1))

		if i == 0:
			t.sides[tertiaryDirectionReverse] = Glue()
			t.sides[secondaryDirection] = Glue(2, "Placement0")
		elif i >= initial and ((i - initial) % vDist) == 0:
			dpIndex = ((i - initial) / vDist)
			t.sides[primaryDirectionReverse] = Glue(2, "{}Place{}".format(dataPaths[dpIndex], 0))
		tdsOut.AddTile(t)

	for i in range(horizontalLeft):
		i = i + 1
		t = TileType(True)
		t.name = "Left{}/{}".format(i, horizontalLeft)
		t.sides[secondaryDirectionReverse] = Glue(2, "Left{}".format(i+1))
		t.sides[secondaryDirection] = Glue(2, "Left{}".format(i))
		if i == 2:
			t.sides[primaryDirection] = Glue(1, "BracketConnectionStart")
		if i == horizontalLeft:
			t.sides[secondaryDirectionReverse] = Glue()
			t.sides[tertiaryDirection] = Glue(2, "Placement0")
		tdsOut.AddTile(t)

	"""t = TileType(True)
	t.name = "Up1/2"
	t.sides[tertiaryDirectionReverse] = Glue(2, "Up1/2")
	t.sides[tertiaryDirection] = Glue(2, "Up2/2")
	t.sides[primaryDirection] = Glue(1, "Guide")
	t.color = "Green"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "Up2/2"
	t.sides[tertiaryDirectionReverse] = Glue(2, "Up2/2")
	t.sides[primaryDirection] = Glue(1, "GuideTurn")
	t.color = "Green"
	tdsOut.AddTile(t)"""

	t = TileType(True)
	t.name = "Up1/2"
	t.sides[tertiaryDirectionReverse] = Glue(2, "Up1/1")
	#t.sides[tertiaryDirection] = Glue(2, "Up2/2")
	t.sides[primaryDirection] = Glue(1, "GuideTurn")
	t.color = "Green"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "Left1/1"
	t.sides[secondaryDirection] = Glue(2, "Left1/1")
	t.sides[primaryDirection] = Glue(1, "GuideHorizontal")
	t.color = "Green"
	tdsOut.AddTile(t)

	for index, dp in enumerate(dataPaths):
		for i in range(totalHorizontal + 1):
			t = TileType(True)
			t.name = "{}Place{}/{}".format(dp, i, totalHorizontal)
			t.sides[secondaryDirectionReverse] = Glue(2, "{}Place{}".format(dp,i))
			t.sides[secondaryDirection] = Glue(2, "{}Place{}".format(dp,i+1))
			if i == 0:
				t.sides[secondaryDirectionReverse] = Glue()
				t.sides[primaryDirection] = Glue(2, "{}Place{}".format(dp, 0))
			if i == totalHorizontal:
				t.sides[secondaryDirection] = Glue()
				t.sides[primaryDirection] = Glue(1, dp)
			tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "{}Prop".format(dp)
		t.sides[primaryDirection] = Glue(1, dp)
		t.sides[primaryDirectionReverse] = Glue(1, dp)
		t.sides[secondaryDirection] = Glue(1, "LSB")
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "{}Guide".format(dp)
		t.sides[primaryDirection] = Glue(1, dp)
		t.sides[primaryDirectionReverse] = Glue(1, dp)
		t.sides[secondaryDirection] = Glue(1, "ZERO")
		t.sides[secondaryDirectionReverse] = Glue(2, "{}GuideStart".format(dp))
		t.color = "Magenta"
		tdsOut.AddTile(t)

		if dp == "N":
			for i in range(initial-2):
				t = TileType(True)
				t.name = "{}Guide{}/{}".format(dp, i, initial-3)
				t.sides[tertiaryDirection] = Glue(2, "{}Guide{}".format(dp, i))
				t.sides[tertiaryDirectionReverse] = Glue(2, "{}Guide{}".format(dp, i+1))
				t.sides[primaryDirection] = Glue(1, "Guide")
				if i == initial-3:
					t.sides[tertiaryDirectionReverse] = Glue()
					t.sides[primaryDirection] = Glue(1, "Final")
				t.color = "Green"
				tdsOut.AddTile(t)

				t = TileType(True)
				t.name = "{}TrueGuide{}/{}".format(dp, i, initial-3)
				t.sides[tertiaryDirection] = Glue(2, "{}TrueGuide{}".format(dp, i))
				t.sides[tertiaryDirectionReverse] = Glue(2, "{}TrueGuide{}".format(dp, i+1))
				t.sides[primaryDirection] = Glue(1, "Guide")
				if i == initial-3:
					t.sides[tertiaryDirectionReverse] = Glue()
					t.sides[primaryDirection] = Glue(1, "NorthFinal")
				if i == 0:
					startTile = t.Copy()
					startTile.name = "{}GuideStart".format(dp)
					#startTile.sides[tertiaryDirection] = Glue(2, "Up1/2")
					startTile.sides[tertiaryDirection] = Glue(2, "Up1/1")
					startTile.sides[secondaryDirection] = Glue(2, "{}GuideStart".format(dp))
					startTile.color = "Magenta"
					tdsOut.AddTile(startTile)
				t.color = "Cyan"
				tdsOut.AddTile(t)
		else:
			for i in range(vDist):
				t = TileType(True)
				t.name = "{}Guide{}/{}".format(dp, i, vDist-1)
				t.sides[tertiaryDirection] = Glue(2, "{}Guide{}".format(dp, i))
				t.sides[tertiaryDirectionReverse] = Glue(2, "{}Guide{}".format(dp, i+1))
				t.sides[primaryDirection] = Glue(1, "Guide")
				if i == vDist - 1:
					t.sides[tertiaryDirectionReverse] = Glue(2, "{}Guide{}".format(dataPaths[index-1], 0))
				if i == 0:
					startTile = t.Copy()
					startTile.name = "{}GuideStart".format(dp)
					#startTile.sides[tertiaryDirection] = Glue(2, "Up1/2")
					startTile.sides[tertiaryDirection] = Glue(2, "Up1/1")
					startTile.sides[secondaryDirection] = Glue(2, "{}GuideStart".format(dp))
					startTile.color = "Magenta"
					tdsOut.AddTile(startTile)
				t.color = "Green"
				tdsOut.AddTile(t)

def adderNumTiles():
	data = ["0", "1", "0l", "1l", "0lm", "1lm", "1m"]
	for digit in data:
		t = TileType(True)
		t.name = "{}_AdderSeed".format(digit)
		t.sides[primaryDirection] = Glue(1, digit)
		if "1" in digit:
			t.color = "Blue"
		tdsOut.AddTile(t)
		tdsSeed.AddTile(t)

	for digit in data:
		t = TileType(True)
		t.name = "{}_PreTurn_Adder".format(digit)
		t.sides[secondaryDirection] = Glue(1, "PreTurn_Adder")
		t.sides[secondaryDirectionReverse] = Glue(1, "PreTurn_Adder")
		t.sides[primaryDirectionReverse] = Glue(1, digit)
		t.sides[primaryDirection] = Glue(1, digit)
		if "1" in digit:
			t.color = "Blue"
		tdsOut.AddTile(t)

	for digit in data:
		t = TileType(True)
		t.name = "{}_Turn_Adder".format(digit)
		t.sides[secondaryDirection] = Glue(1, "Turn_Adder")
		t.sides[secondaryDirectionReverse] = Glue(1, "PreTurn_Adder")
		t.sides[primaryDirectionReverse] = Glue(1, digit)
		t.sides[primaryDirection] = Glue(2, "{}_PassTurn_Adder".format(digit))
		if "1" in digit:
			t.color = "Blue"
		tdsOut.AddTile(t)

	for digit in data:
		t = TileType(True)
		t.name = "{}_PassTurn_Adder".format(digit)
		t.sides[secondaryDirection] = Glue(1, digit)
		t.sides[primaryDirectionReverse] = Glue(2, "{}_PassTurn_Adder".format(digit))
		if "m" not in digit:
			t.sides[secondaryDirectionReverse] = Glue(1, "Turn_Adder")
			t.sides[primaryDirection] = Glue(1, "Prop_Adder")
		if "1" in digit:
			t.color = "Blue"
		tdsOut.AddTile(t)

	for digit in data:
		t = TileType(True)
		t.name = "{}_Prop_Adder".format(digit)
		t.sides[secondaryDirectionReverse] = Glue(1, digit)
		t.sides[secondaryDirection] = Glue(1, digit)
		t.sides[primaryDirectionReverse] = Glue(1, "Prop_Adder")
		if "m" not in digit:
			t.sides[primaryDirection] = Glue(1, "Prop_Adder")
		if "1" in digit:
			t.color = "Blue"
		tdsOut.AddTile(t)

	for digit in data:
		t = TileType(True)
		t.name = "{}_StartRide_Adder".format(digit)
		t.sides[primaryDirectionReverse] = Glue(1, "Ride_Adder")
		t.sides[tertiaryDirectionReverse] = Glue(1, digit)
		if "m" not in digit:
			t.sides[primaryDirection] = Glue(1, "Ride_Adder")
		t.sides[secondaryDirectionReverse] = Glue(1, digit)
		if "1" in digit:
			t.color = "Blue"
		tdsOut.AddTile(t)

	for digit in data:
		t = TileType(True)
		t.name = "{}_Ride_Adder".format(digit)
		t.sides[primaryDirectionReverse] = Glue(1, "Ride_Adder")
		t.sides[tertiaryDirectionReverse] = Glue(1, digit)
		t.sides[tertiaryDirection] = Glue(1, digit)
		if "m" not in digit:
			t.sides[primaryDirection] = Glue(1, "Ride_Adder")
		if "1" in digit:
			t.color = "Blue"
		tdsOut.AddTile(t)



def AdderTDS(directory=""):
	condensedSeedTiles(65)
	generateLayer1Tiles()
	generateLayer2Tiles()
	generateLayer3Tiles()
	generateLayer4Tiles()
	generateAdderTiles()
	generateCheckTiles()
	generateDataPath()
	generateDPPlacementTiles()
	generateDPGuideFollowTiles()
	periodicCounterTiles()
	adderNumTiles()

	tdsSeed.Write(directory)
	return tdsOut

def AdderPrimaryLength(tau):
	return (len(bin(tau).replace("0b","")) + 2) * (numDataPaths+1) * (numAdders+2)

def AdderSecondaryLength(tau):
	spacing = 10
	return len(bin(AdderPrimaryLength).replace("0b", "0")) + 2 + spacing

def twoCompliment(n, tau):
	dataWidth = len(bin(tau).replace("0b","")) + 1

	if n == 0:
		return "0"
	temp = format(int(abs(n)-1), "0{}b".format(dataWidth))
	temp = temp.replace("0","*")
	temp = temp.replace("1","0")
	result = temp.replace("*","1")
	return result

def adderTDP(origin=[0,0,0], tau=2, tileNum=0, numTiles=1):
	dataWidth = len(bin(tau).replace("0b","")) + 1
	dataPeriod = dataWidth + 1
	mainCounterPeriod = (numDataPaths + 1) * dataPeriod
	totalLength = (numAdders+2) * mainCounterPeriod


	adderNumBin = bin(tileNum).replace("0b","")
	adderNumBinTiles = [digit for digit in adderNumBin[::-1]]
	adderNumBinTiles[0] += "l"
	adderNumBinTiles[-1] += "m"
	
	print adderNumBinTiles

	tauRepresentation = []
	tauBinary = twoCompliment(tau, tau)
	tauRepresentation = [digit for digit in tauBinary][::-1]
	#tauRepresentation[0] += "l"
	tauRepresentation[-1] += "m"



	#print totalLength
	PC_0 = PeriodicCounter(0, PCoffset-1)
	PC_1 = PeriodicCounter(1, PCoffset-1)
	PC_2 = PeriodicCounter(2, PCoffset-1)

	result = TDP("Adder{}".format(tau), tdsOut.name)

	result.AddTile("MainCounterSeed0", XYZ([0,0,0], "NU"))
	result.AddTile("MainCounterSeed1", XYZ([0,1,0], "NU"))

	for i, data in enumerate(adderNumBinTiles):
		result.AddTile("{}_AdderSeed".format(data), [-3-i, 0, 0])

	for i, data in enumerate(tauRepresentation):
		result.AddTile("{}_AdderSeed".format(data), [3+i, -7, 91])

	offset = -1*mainCounterPeriod+1
	result.Merge(PC_0.seed(totalLength-1, [0,offset,-3*PCoffset]))
	result.Merge(PC_1.seed(mainCounterPeriod-1, [0,offset,-2*PCoffset]))
	result.Merge(PC_2.seed(dataPeriod-1, [0,offset,-1*PCoffset]))

	for i in range(len(dataPaths)):
		#print dataPaths[i]
		x = 3
		y = -1 * mainCounterPeriod + (i * dataPeriod) + 1
		z = 5 + i * (2*PCoffset + verticalOffset)
		result.Merge(PC_0.seed(totalLength-1, [x,y,z]))
		result.Merge(PC_1.seed(mainCounterPeriod-1, [x,y,z+PCoffset]))

	for TL in result.TileAndLocationList:
		TL.location = AddLoc(TL.location, origin)

	return result



def DataPathTDP(inputDict=inputs, origin=[0,0,0], tau=2):
	result = TDP("Adder:{}_DP:{}".format(origin, "".join(inputDict.keys())))
	result.tdsFile = tdsOut.name

	dataWidth = len(bin(tau).replace("0b","")) + 1
	dataPeriod = dataWidth + 1
	mainCounterPeriod = (numDataPaths + 1) * dataPeriod
	totalLength = (numAdders+2) * mainCounterPeriod

	NorthBaseLocation = [3, -1 * mainCounterPeriod, 5+2*PCoffset]
	for dp, value in inputDict.items():
		if value == 0:
			continue
		if dp == "Tau":
			binary = twoCompliment(value, tau)
		else:
			binary = format(int(abs(value)), "0{}b".format(dataWidth))
		binary = binary[::-1]
		#print "{}\t{}\t{}".format(dp, value, binary)

		for i in range(len(binary)):
			if i == 0:
				variant = "LSB"
			elif i == dataWidth-1:
				variant = "MSB"
			else:
				variant = "REG"
			name = "DPData{}{}".format(variant, binary[i])
			x = NorthBaseLocation[0] + i
			y = NorthBaseLocation[1] + dataPaths.index(dp) * dataPeriod
			z = NorthBaseLocation[2] + dataPaths.index(dp) * (2*PCoffset + verticalOffset)

			result.AddTile(name, [x,y,z])

	for TL in result.TileAndLocationList:
		TL.location = AddLoc(TL.location, origin)

	return result

def example(tau):
	tdp = adderTDP([0,0,0], tau=tau, tileNum = 5)
	"""tdp.Merge(
		adderTDP(
			XYZ([0,25,0], "NU")
		)
	)"""
	tdp.Merge(
		DataPathTDP(
			inputDict = {"N":2, "E":1}, 
			origin = XYZ([0,0,0], "NU"),
			tau = tau,
		)
	)
	tdp.AddTile("0_AdderSeed", [-2, 1, -50])
	"""tdp.Merge(
		DataPathTDP(
			inputDict = {"D":2, "Tau":2}, 
			origin = XYZ([0,25,0], "NU")
		)
	)"""

	temp = []
	for TL in tdp.TileAndLocationList:
		if TL.location[2] < 1:
			temp.append(TL)
	#tdp.TileAndLocationList = temp
	tdp.name = "Adder.tdp"
	tdp.Write("Adder_Example")
	AdderTDS("Adder_Example").Write("Adder_Example")

example(2)
tdp = adderTDP([0,0,0], 2, tileNum=5, numTiles=7)
tdp.Write("Adder_Example")
tdp = adderTDP([0,0,0], 4, tileNum=5, numTiles=7)
tdp.Write("Adder_Example")
tdp = adderTDP([0,0,0], 10, tileNum=5, numTiles=7)
tdp.Write("Adder_Example")
tdp = adderTDP([0,0,0], 100, tileNum=5, numTiles=7)
tdp.Write("Adder_Example")
tdp = adderTDP([0,0,0], 100000, tileNum=5, numTiles=7)
tdp.Write("Adder_Example")


#condensedSeed()
#mainCounterPeriodicCounters()

"""tdsOut.Write()
tdsSeed.Write()

tdpOut.Write()
tdpSeed.TileAndLocationList = [TL for TL in tdpOut.TileAndLocationList if tdsSeed.tileDictionary.get(TL.tileName)]
tdpSeed.Write()
"""