# This script generates a tdp that will create a datapath that will grow into the adder with the specified proportions

import sys
from itertools import izip

# pairwise iterator
def pairwise(iterable):
    a = iter(iterable)
    return izip(a, a)

# returns an array of tiles that represent a movement of n tiles forward
def forward(n):
    if 0 < n <= 3:
        return ['Data'] * n
    if n > 3:
        path = ['FW']
        d = int((n - 3) / 2) + 1
        bits = "{0:b}".format(d)
        for b in reversed(bits):
            path.append('C'+b)
        path.append('MSB')
        if n % 2 == 0:
            path.append('Data')
        return path
    else:
        return None

# generates placement instructions from row of pairs
def placement(row):
    path = []
    if len(row) % 2 == 1:
        path.append('Place-{' + str(0) + '-' + str(row[0]) + '}')
        # print "PAIR:", 0, row[0]

        for a, b in pairwise(row[1:]):
            path.append('Place-{' + str(a) + '-' + str(b) + '}')
            # print "PAIR:", a, b
    else:
        for a, b in pairwise(row):
            path.append('Place-{' + str(a) + '-' + str(b) + '}')
            # print "PAIR:", a, b
    
    return path

# converts the isntruction and position to a tile name and position
def tile_name(inst):
    s = inst + "-Start-DN-0"
    return s

# generates the actual instruction tiles from the paths array
# note there will be a space between each one
def generate_paths(paths, padded):
    instructions = []
    tiles_list = []

    for path in paths:
        tiles = []
        tiles.append(tile_name('RB'))
        tiles.append(tile_name('Inst'))

        for inst in path:
           tiles.append(tile_name(inst))

        for i in range(padded - len(path) - 3):
            tiles.append(tile_name('Data'))

        tiles.append(tile_name('LB'))
        tiles_list.append(tiles)

    for t in tiles_list:
        for i in reversed(t):
            instructions.append(i)
        # instructions.append('Gap')
    # print instructions
    return instructions

def adder_padded_length(m, T, tau):
    return 3 * m.bit_length() + T * (6 * tau.bit_length() + (11 + 3 * (m + tau).bit_length()).bit_length())

def generate_adder_initialization(paddedLength, scaleFactor, startingOffset, spacing, adderCount, xOffset, yOffset, zOffsetMajor, zOffsetMinor, rows, xCounterOffset, yCounterOffset, downDistance, northDistance, westDistance):

    seedIds = {
        'Blank'             : 0,
        'MainCounterSeed0'  : 1,
        'MainCounterSeed1'  : 2,
        'Seed:1m|1m:prop'   : 3,
        'Seed:1m|1m:nb'     : 4,
        'Seed:1m|1m:b'      : 5,
        'Seed:1r|1r:nb'     : 6,
        'Seed:1r|1r:b'      : 7,
        'Seed:1r|1r:prop'   : 8,
        'Seed:0r|0r:nb'     : 9,
        'Seed:0r|0r:b'      : 10,
        'Seed:0r|0r:prop'   : 11,
        'Seed:1l|1l:layer0' : 12,
        'Seed:0l|0l:layer0' : 13,
        'Seed:1l|1l:layer1' : 14,
        'Seed:0l|0l:layer1' : 15,
        'Seed:1l|1l:layer2' : 16,
        'Seed:0l|0l:layer2' : 17,
        '0_AdderSeed'       : 18,
        '1_AdderSeed'       : 19,
        '0l_AdderSeed'      : 20,
        '1l_AdderSeed'      : 21,
        '0lm_AdderSeed'     : 22,
        '1lm_AdderSeed'     : 23,
        '1m_AdderSeed'      : 24,
        'Blank_AdderSeed'   : 25,
    }

    idRows = []
    for i in range(3):
        idRow = []
        for name in rows[i]:
            idRow.append(seedIds[name])
        idRow.reverse()
        idRows.append(idRow)

    rowOffsets = []
    for row in rows:
        if len(row) % 2 == 1:
            rowOffsets.append(0)
        else:
            rowOffsets.append(1)

    # this array will store all of the instructions for the multiple datapaths
    paths = []

    # this compensates for the datapaths before this one
    pathOffset = 0
    zOffset = 0

    downDistance = downDistance - 2
    northDistance = northDistance - paddedLength - 3
    westDistance = westDistance - 1 + startingOffset

    path = []
    path.extend(forward(downDistance))
    path.append('Rise')
    path.extend(forward(northDistance))
    path.append('LT')
    path.extend(forward(westDistance + rowOffsets[0]))
    for i in range(adderCount):
        if i > 0:
            path.extend(forward(spacing - len(rows[0]) + rowOffsets[0] - 1))
        path.extend(placement(idRows[0]))
    paths.append(path)

    downDistance -= zOffsetMinor
    westDistance += paddedLength

    path = []
    path.extend(forward(downDistance))
    path.append('Rise')
    path.extend(forward(northDistance))
    path.append('LT')
    path.extend(forward(westDistance + rowOffsets[1]))
    for i in range(adderCount):
        if i > 0:
            path.extend(forward(spacing - len(rows[0]) + rowOffsets[1] - 1))
        path.extend(forward(len(rows[0]) - len(rows[1])))
        path.extend(placement(idRows[1]))
    paths.append(path)

    downDistance -= zOffsetMinor
    westDistance += paddedLength

    path = []
    path.extend(forward(downDistance))
    path.append('Rise')
    path.extend(forward(northDistance))
    path.append('LT')
    path.extend(forward(westDistance + rowOffsets[2]))
    for i in range(adderCount):
        if i > 0:
            path.extend(forward(spacing - len(rows[0]) + rowOffsets[2] - 1))
        path.extend(forward(len(rows[0]) - len(rows[2])))
        path.extend(placement(idRows[2]))
    paths.append(path)

    westDistance += paddedLength

    # TODO handle more general case
    adderIds = [[20, 18, 24, 25], [21, 18, 24, 25], [20, 19, 24, 25], [21, 19, 24, 25]]
    # just = (adderCount-1).bit_length()
    # for i in range(adderCount):
    #     bits = bin(i)[2:].rjust(just, '0')
    #     tiles = []
    #     if just == 1:
    #         tiles = [22 + int(bits), 25]
    #     else:
    #         i = 0
    #         for bit in bits:

    print xCounterOffset

    path = []
    path.extend(forward(downDistance - zOffsetMinor))
    path.append('Rise')
    path.extend(forward(northDistance + yCounterOffset))
    path.append('LT')
    path.extend(forward(westDistance + xCounterOffset + 1))
    for i in range(adderCount):
        placeLength = len(adderIds[i]) + 4
        if i > 0:
            path.extend(forward(spacing - placeLength))
        # path.extend(forward(len(rows[0]) - 4 + len(adderIds[i])))
        path.extend(placement([2, 1]))
        path.extend(forward(2))
        path.extend(placement(adderIds[i]))
    paths.append(path)

    westDistance -= xOffset

    for k in range(7):
        downDistance -= zOffsetMajor
        westDistance += paddedLength

        pathOffset += paddedLength
        zOffset += zOffsetMajor

        path = []
        path.extend(forward(downDistance))
        path.append('Rise')
        path.extend(forward(northDistance))
        path.append('LT')
        path.extend(forward(westDistance + rowOffsets[0]))
        for i in range(adderCount):
            if i > 0:
                path.extend(forward(spacing - len(rows[0]) - 1))
            path.extend(placement(idRows[0]))
        paths.append(path)

        westDistance += paddedLength
        downDistance -= zOffsetMinor

        path = []
        path.extend(forward(downDistance))
        path.append('Rise')
        path.extend(forward(northDistance))
        path.append('LT')
        path.extend(forward(westDistance + rowOffsets[1]))
        for i in range(adderCount):
            if i > 0:
                path.extend(forward(spacing - len(rows[0]) - 1))
            path.extend(forward(len(rows[0]) - len(rows[1])))
            path.extend(placement(idRows[1]))
        paths.append(path)

        northDistance += yOffset

    # # Place Tau Row

    westDistance += paddedLength
    downDistance -= zOffsetMinor

    tau = [25, 24, 19, 18]
    tauOffset = 1 - (len(tau) % 2)

    path = []
    path.extend(forward(downDistance))
    path.append('Rise')
    path.extend(forward(northDistance))
    path.append('LT')
    path.extend(forward(westDistance + tauOffset))
    for i in range(adderCount):
        if i > 0:
            path.extend(forward(spacing - len(rows[0])))
        path.extend(forward(len(rows[0]) - len(tau)))
        path.extend(placement(tau))
    paths.append(path)

    instructions = generate_paths(paths, paddedLength)
    return instructions
